package com.sheelapps.swtml.examples.texteditor;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Vector;

import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.StyleRange;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.MenuItem;

public class FileMenuListener implements Listener {

	public void handleEvent(Event event) {
		MenuItem item = (MenuItem) event.widget;
		if (item.getText().equalsIgnoreCase("New")) {
			TextEditor.getStyledText().setText("");
			TextEditor.fileName = null;
			TextEditor.shell.setText("Text Editor - Untitled");
		} else if (item.getText().equalsIgnoreCase("Open")) {
			open();
		} else if (item.getText().equalsIgnoreCase("Save")) {
			save();
		} else if (item.getText().equalsIgnoreCase("Exit")) {
			TextEditor.dispose();
		}

	}

	private void open() {
		FileDialog dlg = new FileDialog(TextEditor.shell, SWT.OPEN);
		String file = dlg.open();
		if (file != null) {
			File localFile = new File(file);
			if (localFile != null && !localFile.isDirectory()) {
				BufferedReader reader;
				TextEditor.fileName = localFile.getAbsolutePath();
				try {
					reader = new BufferedReader(new FileReader(localFile));
					parseData(reader);
					TextEditor.shell.setText("Text Editor-"
							+ TextEditor.fileName);

				} catch (FileNotFoundException e) {
					e.printStackTrace();
				}
			}

		}

	}

	private void parseData(BufferedReader reader) {

		try {

			String currLine = null;
			boolean style = false;
			StringBuffer data = new StringBuffer();
			Vector styles = new Vector();
			while (((currLine = reader.readLine()) != null)) {
				if (TextEditor.START_STYLES_MARK.equals(currLine)) {
					style = true;
				} else {
					if (!style)
						data.append(currLine + TextEditor.newline);
					if (style)
						styles.add(currLine);
				}
			}
			TextEditor.getStyledText().setText(data.toString());

			applyStyle(styles);

		} catch (Exception e) {
			e.printStackTrace();
		} finally {

			try {
				if (reader != null)
					reader.close();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
	}

	private void applyStyle(Vector styles) {
		StyleRange range = null;
		String st = null;
		for (int i = 0; i < styles.size(); i++) {
			st = (String) styles.get(i);
			range = getStyle(st);
			if (range != null)
				TextEditor.getStyledText().setStyleRange(range);
		}

	}

	private StyleRange getStyle(String styleStr) {
		StyleRange styleRange = new StyleRange();
		if (styleStr.startsWith("[[FONT ")) {
			String fontData = styleStr
					.substring(styleStr.indexOf("[[FONT ") + 7);
			FontData fdata = new FontData(fontData);
			Font f = new Font(Display.getCurrent(), fdata);
			TextEditor.getStyledText().setFont(f);
		} else if (styleStr.startsWith("[StyleRange {")) {

			String startpos = styleStr.substring(
					styleStr.indexOf("[StyleRange {") + 13,
					styleStr.indexOf(",")).trim();
			String endPos = styleStr.substring(styleStr.indexOf(",") + 1);
			endPos = endPos.substring(0, endPos.indexOf(",")).trim();
			styleRange.start = Integer.parseInt(startpos);
			styleRange.length = Integer.parseInt(endPos);
			String fontStyle = styleStr.substring(styleStr
					.indexOf("fontStyle="));
			if (fontStyle.indexOf("bold") >= 0) {
				styleRange.fontStyle = styleRange.fontStyle | SWT.BOLD;
			}
			if (fontStyle.indexOf("underlined") >= 0) {
				styleRange.underline = true;
			}
			if (fontStyle.indexOf("striked out") >= 0) {
				styleRange.strikeout = true;
			}
			if (fontStyle.indexOf("italic") >= 0) {
				styleRange.fontStyle = styleRange.fontStyle | SWT.ITALIC;
			}
			if (styleStr.indexOf("foreground=Color {") >= 0) {
				String color = styleStr.substring(styleStr
						.indexOf("foreground=Color {") + 17);
				color = color.substring(0, color.indexOf("}"));
				String red = color.substring(color.indexOf("{") + 1,
						color.indexOf(",")).trim();
				String green = color.substring(color.indexOf(",") + 1);
				green = green.substring(0, green.indexOf(",")).trim();
				String blue = color.substring(color.lastIndexOf(",") + 1)
						.trim();
				styleRange.foreground = new Color(null, Integer.parseInt(red),
						Integer.parseInt(green), Integer.parseInt(blue));
			}

		}

		return styleRange;
	}

	private void save() {
		PrintWriter print = null;
		try {
			getFileName();
			if (TextEditor.fileName == null) {
				return;
			}
			String textData = TextEditor.getStyledText().getText();
			print = new PrintWriter(new FileWriter(TextEditor.fileName));
			print.println(textData);
			StyleRange[] styles = TextEditor.getStyledText().getStyleRanges();
			if (styles.length > 0) {
				print.println(TextEditor.START_STYLES_MARK);

				for (int i = 0; i < styles.length; i++) {
					print.println("[" + styles[i].toString() + "]");

				}
				print.println("[[FONT "
						+ TextEditor.getStyledText().getFont().getFontData()[0]
								.toString());
			}
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} finally {
			if (print != null)
				print.close();
		}

	}

	private void getFileName() {
		if (TextEditor.fileName == null) {
			SafeSaveDialog dlg = new SafeSaveDialog(TextEditor.shell);
			String f = dlg.open();
			if (f != null) {
				TextEditor.fileName = f;
				TextEditor.shell.setText("Text Editor-" + TextEditor.fileName);
			}
		}

	}

}
